/*
** Copyright 2022 Centreon
**
** Licensed under the Apache License, Version 2.0 (the "License");
** you may not use this file except in compliance with the License.
** You may obtain a copy of the License at
**
**     http://www.apache.org/licenses/LICENSE-2.0
**
** Unless required by applicable law or agreed to in writing, software
** distributed under the License is distributed on an "AS IS" BASIS,
** WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
** See the License for the specific language governing permissions and
** limitations under the License.
**
** For more information : contact@centreon.com
*/

syntax = "proto3";

package com.centreon.broker;

/* Ignore */
message Bbdo {
  uint32 major = 1;
  uint32 minor = 2;
  uint32 patch = 3;
}

/* io::bbdo, bbdo::de_welcome, 49 */
message Welcome {
  Bbdo version = 1;
  string extensions = 2;
  uint64 poller_id = 3;
  string poller_name = 4;
}

/* io::bbdo, bbdo::de_pb_ack, 50 */
message Ack {
  uint32 acknowledged_events = 1;
}

/* io::bbdo, bbdo::de_pb_stop, 51 */
message Stop {
  uint64 poller_id = 1;
}
