/*
 * Copyright 2023 Centreon (https://www.centreon.com/)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * For more information : contact@centreon.com
 *
 */

#include "customvariable.hh"

using namespace com::centreon::engine::configuration;

customvariable::customvariable(const std::string& value, bool is_sent)
    : _value{value}, _is_sent{is_sent} {}

// customvariable::customvariable(customvariable const& other)
//     : _value{other._value}, _is_sent{other._is_sent} {}

// customvariable& customvariable::operator=(const customvariable& other) {
//   if (this != &other) {
//     _value = other.value;
//     _is_sent = other._is_sent;
//   }
//   return *this;
// }

void customvariable::set_sent(bool sent) {
  _is_sent = sent;
}

bool customvariable::is_sent() const {
  return _is_sent;
}

void customvariable::set_value(const std::string& value) {
  _value = value;
}

const std::string& customvariable::value() const {
  return _value;
}

bool customvariable::operator==(const customvariable& other) const {
  return _value == other._value && _is_sent == other._is_sent;
}
